#include "R4SKetlle.h"


R4SKetlleStatus::R4SKetlleStatus(uint8_t* data) {
  m_valid = (data != 0);
  if (!m_valid)
    return;

  program = *(data + 3);
  targetTemperature = *(data + 5);
#ifdef R4S_CUSTOM_STATE
  remainingHours = *(data + 7);
  currentTemperature = *(data + 8);
#else
  remainingHours = *(data + 8);
  currentTemperature = *(data + 13);
#endif
  remainingMinutes = *(data + 9);
  isHeating = *(data + 10) == 1;
  state = *(data + 11);
  error = *(data + 12);
}

R4SKetlleStatus::R4SKetlleStatus(const R4SKetlleStatus& other) {
  copyFrom(other);
}

R4SKetlleStatus& R4SKetlleStatus::operator =(const R4SKetlleStatus& other) {
  copyFrom(other);
  return *this;
}


bool R4SKetlleStatus::isValid() {
  return m_valid;
}

void R4SKetlleStatus::copyFrom(const R4SKetlleStatus& other) {
  m_valid = other.m_valid;
  program = other.program;
  targetTemperature = other.targetTemperature;
  remainingHours = other.remainingHours;
  remainingMinutes = other.remainingMinutes;
  isHeating = other.isHeating;
  error = other.error;
  currentTemperature = other.currentTemperature;
}

String getStatusJson(BLEAddress* addr) {
  String json;
  R4SKetlleStatus st = r4SKetlleStatus();
  if (st.isValid()) {
    json = String("{");
    json += String("\"temp\":") + String(st.currentTemperature) + String(",");
    json += String("\"target\": ") + String(st.targetTemperature) + String(",");
    json += String("\"heat\": ") + String(st.isHeating ? "1" : "0") + String(",");
    json += String("\"state\": ") + String(st.state) + String(",");
    json += String("\"hours\": ") + String(st.remainingHours) + String(",");
    json += String("\"mins\": ") + String(st.remainingMinutes) + String(",");
    json += String("\"prog\": ") + String(st.program) + String(",");
    json += String("\"error\": ") + String(st.error);
    json += String("}");
  }
  return json;
}

bool r4SKetlleOff() {
  if (r4sCommand(0x04) != 5)
    return false;
  return notifyData[3] == 1;
}

//bool r4SKetlleOn(bool boil, uint8_t temp) {
bool r4SKetlleOn(uint8_t prog, uint8_t temp) {
#ifdef R4S_CUSTOM_STATE
  //uint8_t data[] = { boil ? (uint8_t)0 : (uint8_t)1, 0, temp, 0, 1, temp, 30, 0, 0, 0, 0, 0, 0, 128, 0, 0};
  uint8_t data[] = { prog, 0, temp, 0, 1, temp, 30, 0, 0, 0, 0, 0, 0, 128, 0, 0};
#else
  //uint8_t data[] = { boil ? (uint8_t)0 : (uint8_t)1, 0, temp, 0 };
  uint8_t data[] = { prog, 0, temp, 0 };
#endif
  if (r4sCommand(0x05, data, sizeof(data)) != 5)
    return false;
#ifdef R4S_CUSTOM_STATE
  if (notifyData[3] != 1)
#else
  if (notifyData[3] == 1)
#endif
    return false;

#ifdef R4S_2STEP_ON
  if (r4sCommand(0x03) != 5)
    return false;
  if (notifyData[3] != 1)
    return false;
#endif

  return true;
}

bool r4SKetlleBoil() {
  //return r4SKetlleOn(true, 0);
  return r4SKetlleOn(0, 0);
}

bool r4SKetlleHeat(uint8_t temp) {
  //return r4SKetlleOn(false, temp);
  return r4SKetlleOn(1, temp);
}

bool r4SKetlleBoilAndHeat(uint8_t temp) {
  //return r4SKetlleOn(true, temp);
  return r4SKetlleOn(2, temp);
}

#ifdef R4S_NIGHT_LIGHT
bool r4SKetlle_NLOn() {
  if (r4sCommand(0x32, nl_settings, sizeof(nl_settings)) != 5)
    return false;
  if (notifyData[3] == 1)
    return false;
    
  uint8_t data[] = { 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128, 0, 0};
  if (r4sCommand(0x05, data, sizeof(data)) != 5)
    return false;
  if (notifyData[3] == 1)
    return false;
    
  if (r4sCommand(0x03) != 5)
    return false;
  if (notifyData[3] != 1)
    return false;
    
  return true;
}

bool r4SKetlle_NLsettings() {
  uint8_t data[] = { 1 };
  uint8_t resp = r4sCommand(0x33, data, sizeof(data));
  Serial.println(resp);

  return true;
}
#endif

R4SKetlleStatus r4SKetlleStatus() {
  if (r4sCommand(0x06) != 20)
    return R4SKetlleStatus();

  return R4SKetlleStatus(notifyData);
}
